package board;

import java.awt.event.*;
import java.awt.*;
import java.applet.*;
import java.lang.Math.*;
import java.util.*;


public class DrawPanel extends Panel implements ParameterListener{

	DrawPanel(){
		MouseDispatcher mouseDispatcher = new MouseDispatcher();
		addMouseListener(mouseDispatcher);
		addMouseMotionListener(mouseDispatcher);
		
		this.addComponentListener(new ComponentAdapter(){
			public void componentResized(ComponentEvent e){
				resized();
			}
		});
	}
	
	
	
	public void clear(){
		record.clear();
		view.position = 0;
		System.out.println("paint");
		repaint_all();
	}
	
	
	
	public void save(){
		Frame frame = (Frame)this.getParent();
		java.awt.FileDialog fileDialog = new java.awt.FileDialog( frame, "Save", java.awt.FileDialog.SAVE );
		fileDialog.setDirectory("..\\data\\");
		fileDialog.setFile("*.txt");
		fileDialog.setVisible(true);
		if( fileDialog.getFile()!=null )
		{
			String filename = fileDialog.getDirectory() + fileDialog.getFile();
			record.save(filename);
		}
		fileDialog.dispose();
	}

	public void load(){
		Frame frame = (Frame)this.getParent();
		java.awt.FileDialog fileDialog = new java.awt.FileDialog( frame, "Load", java.awt.FileDialog.LOAD );
		fileDialog.setDirectory("..\\data\\");
		fileDialog.setFile("*.txt");
		fileDialog.setVisible(true);
		if( fileDialog.getFile()!=null )
		{
			String filename = fileDialog.getDirectory() + fileDialog.getFile();
			record.load(filename);
		}
		fileDialog.dispose();
		
		repaint_all();
	}
	
	
	
	static public int pen_width = 6;
	public void width(){
		new ParameterDialog((Frame)this.getParent(), this, "width", ""+pen_width);	
	}
	public void set(String name, String value){
		if (name.equals("width")){
			pen_width = (new Integer(value)).intValue();
			basicStroke = new BasicStroke(pen_width, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
		}
	}
					
		
	
	

	
	

	Image buffered_image = null;
	public static BasicStroke basicStroke = new BasicStroke(pen_width, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
	public static BasicStroke defaultStroke = new BasicStroke();
	boolean repaint_all;
	public void repaint_all(){
		repaint_all = true;
		repaint();
	}
	public void update(Graphics g) {
		paint(g);
	}
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		//Graphics g2 = g;

		g2.setStroke(basicStroke);
		//g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		if (repaint_all){
			
		//double buffering
		if (buffered_image == null)
			buffered_image = createImage(getSize().width, getSize().height);
			Graphics2D bg = (Graphics2D) buffered_image.getGraphics();
			bg.setStroke(basicStroke);
			//bg.setStroke(basicStroke);
			//bg.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

			
			// clear
			bg.clearRect(0,0, getSize().width, getSize().height);

			// paint code!
			//record.paint(bg);
			view.paint(bg);

			if (lasso != null)
				lasso.paint(bg);

		
		
			//double buffering
			bg.dispose();
      
			g2.drawImage(buffered_image,0,0,this);
		
			repaint_all = false;
		}

		if (toolPanel != null)
			toolPanel.paint(g);
		if (toolSelector.active){
			g2.setStroke(defaultStroke);
			toolSelector.paint(g);
		}
				
		if (stroke != null){
			stroke.paint(g2);
			if (clear_stroke){
				clear_stroke = false;
				stroke = null;
			}
		}

	}
	public void resized(){
		buffered_image = null;
		repaint_all();
	}
   
   
	
	
	
	
	
	
	
	public Record record = new Record();
	public View view = new View(record);	
	
	
	//Vector strokes = new Vector();
	Stroke stroke = null;
	public void start_stroke(Point p){
		stroke = new Stroke();
		stroke.width = pen_width;
		extend_stroke(p);
	}
	public void extend_stroke(Point p){
		stroke.addPoint(p);
		repaint();
	}
	boolean clear_stroke;
	public void finish_stroke(Point p){
		view.addStroke(stroke);
		clear_stroke = true;
		repaint();
		
	}


	
	
	
	
	
	
	Lasso lasso;
	public void finish_control_stroke(Point p){
		if (lasso != null){
			if (lasso.stroke.intersects(stroke))
				lasso = null;
			else {
				lasso.release();
				lasso = null;
			}
		}
		else {
			double l =  stroke.length();
			double d = stroke.direct_length();
			
			// Click
			if (l < 10 ){
				toolSelector.activate(p);
			}
			// loop
			else if (l/d > 2) {
				lasso = new Lasso(this, stroke);
			}
		}
		
		stroke = null;
		
		repaint_all();
	}

	
	
	
	
	
	Point prev_p;
	public void start_scrolling(Point p){
		prev_p = p;
	}
	public void scrolling(Point p){
		view.position += prev_p.y - p.y;
		prev_p = p;
		
		repaint_all();
	}
	
	
	
	
	
	
	
	
	
	public void start_dragging(Point p){
		prev_p = p;
	}
	public void dragging(Point p){
		int dx = p.x-prev_p.x;
		int dy = p.y-prev_p.y;
		
		lasso.drag(dx, dy);

		prev_p = p;
		repaint_all();
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	ToolPanel toolPanel;
	ToolSelector toolSelector = new ToolSelector();

	public void clickToolSelector(Point p){
		toolPanel = toolSelector.click(p, this);	
		repaint_all();
		
	}
	
	
	
	
	
	
	
   
   
	// central event dispatcher
	public class MouseDispatcher 
		extends MouseAdapter implements MouseMotionListener{

		int operation_status;
		final int NONE = 0;
		final int DRAWING = 1;
		final int SCROLLING = 2;
		final int CONTROL = 3;
		final int DRAGGING = 4;

  		public void mouseMoved(MouseEvent e) {
	    	e.consume();
  		}
  		public void mouseDragged(MouseEvent e) {
			if (toolPanel != null){
				toolPanel.mouseDragged(e);
				repaint();
				return;
			}

			Point p = e.getPoint();
			
			switch(operation_status){
			case DRAWING:
			case CONTROL:
				extend_stroke(p);
				break;
			case SCROLLING:
				scrolling(p);
				break;
			case DRAGGING:
				dragging(p);
			}
   		}
  		public void mousePressed(MouseEvent e) {
			boolean right_button = (e.getModifiers() & e.BUTTON3_MASK) != 0;
			Point p = e.getPoint();

			if (toolPanel != null){
				toolPanel.mousePressed(e);
				repaint();
				return;
			}
			
			if (toolSelector.active){
				clickToolSelector(p);
				return;
			}
			
			if (lasso != null && lasso.stroke.enclose(p)){
				operation_status = DRAGGING;
				start_dragging(p);
			}
			else if (right_button || lasso != null){
				operation_status = CONTROL;
				start_stroke(e.getPoint());
				stroke.color = Color.red;
			}
			else if (p.x > 1024-View.W){
				operation_status = SCROLLING;
				start_scrolling(e.getPoint());
			}
			else {
				operation_status = DRAWING;
				start_stroke(e.getPoint());
			}
		}
		public void mouseReleased(MouseEvent e) {
			if (toolPanel != null){
				toolPanel.mouseReleased(e);
				repaint();
				return;
			}

			Point p = e.getPoint();

			switch(operation_status){
			case CONTROL:
				finish_control_stroke(p);
				break;
			case DRAWING:
				finish_stroke(p);
				break;
			case SCROLLING:
				break;
			}
			operation_status = NONE;
		}
    }
}







