package board;

import java.awt.*;
public class Vector2D {
	
	public double x;
	public double y;

	Vector2D(Point start, Point end){
		x = end.x -  start.x;
		y = end.y -  start.y;
	}
	Vector2D(Vector2D start, Vector2D end){
		x = end.x -  start.x;
		y = end.y -  start.y;
	}

	Vector2D(Point p){
		x = p.x;
		y = p.y;
	}
	Vector2D(){
	}
	public Vector2D(double _x, double _y){
		x = _x;
		y = _y;
	}
    	
   	Vector2D(Vector2D v) {
	    x = v.x;	
	    y = v.y;
    }	


	public String toString(){
		return "("+x+","+y+")";
	}
	public Vector2D duplicate(){
		return new Vector2D(x,y);
	}



    public static Vector2D normalize(Vector2D v){
		double l = v.length();
		return new Vector2D(v.x/l, v.y/l);
    }
    public void normalize(){
		double l = length();
		x /= l;
		y /= l;
    }
	
	public void negate(){
		x*=-1;y*=-1;
	}
	public static Vector2D negate(Vector2D v){
		return new Vector2D(-v.x, -v.y);	
	}


	
	
	
    public double length(){
		return Math.sqrt(x * x + y * y);
    }
    
	
	
	
	static public Vector2D add(Vector2D u, Vector2D v){
		return new Vector2D(u.x + v.x, u.y + v.y);
    }
    public void add(Vector2D v){
		x += v.x;	y += v.y;	
    }

	static public Vector2D subtract(Vector2D u, Vector2D v){
		return new Vector2D(u.x - v.x, u.y - v.y);
    }
    public void subtract(Vector2D v){
		x -= v.x;	y -= v.y;
    }
	
	
    static public Vector2D multiply(Vector2D v, double m){
		return new Vector2D(v.x * m, v.y * m);
    }
    public void multiply(double m){
		x *= m;	y *= m;	
    }
	
	
	
	
	
	

    static public double dot_product(Vector2D u, Vector2D v){
		return u.x * v.x + u.y * v.y;
    }
	

	
	static public double cos(Vector2D u, Vector2D v){
	  double length = u.length() * v.length();

	  if (length >0)
	    return dot_product(u, v) / length;
	  else
	    return 0;
    }
	
    static public double cross_product(Vector2D u, Vector2D v){
		return u.x * v.y - u.y * v.x;
    }
	
	
    static public double sin(Vector2D u, Vector2D v){
	  double length = u.length() * v.length();

	  if (length >0)
	    return (cross_product(u, v)) / length;
	  else
	    return 0;
    }

	
	
	
   // n_vƂ̋p߂B0-PI Polygon2.set_normal()
    public static double get_angle_PI(Vector2D u, Vector2D v){
		double cosine = cos(u, v);
		if (cosine <= -1) return Math.PI;
		else if (cosine >= 1) return 0;
		else return Math.acos(cosine);
    }	
	
	
	
	
	/** 0-360 n_vꍇ̋p*/
	public static double get_angle_360(Vector2D u, Vector2D v){
		double cos = cos(u, v);
		double sin = sin(u, v);
		if (cos == 0)
			if (sin > 0)	return 90;
			else		return 270;
		if (sin == 0)
			if (cos > 0)	return 0;
			else		return 180;
		
		// -PI/2   ...   +PI/2
		double angle = 180 * Math.atan(sin/cos) / Math.PI;

		if (cos < 0)
			angle = angle + 180;
		if (angle < 0)
			angle = angle + 360;
		return angle;
	}	
	
	// get angel between -180 ... 180
	public double get_angle180(Vector2D v){
		double cos = cos(this,v);
		double sin = sin(this,v);
		if (cos == 0)
			if (sin > 0)	return 90;
			else		return -90;
		if (sin == 0)
			if (cos > 0)	return 0;
			else		return 180;
		
		// -PI/2   ...   +PI/2
		double angle = 180 * Math.atan(sin/cos) / Math.PI;

		if (cos < 0)
			if (angle>0)
				angle -= 180;
			else
				angle +=180;
		return angle;
	}
	
	
	
	
	public static Vector2D rotate90(Vector2D vec){
		return new Vector2D(vec.y, -vec.x);
	}
	
	
	// rotate 0-360
	public static Vector2D rotate(Vector2D v, double degree){
	    if (degree == 90)
		return new Vector2D(-v.y, v.x);
	    else if (degree == 180)
		return new Vector2D(-v.x, -v.y);
	    else if (degree == 270)
		return new Vector2D(v.y, -v.x);

	    
	    double radian = degree * Math.PI / 180.0;
	    double cos = Math.cos(radian);
	    double sin = Math.sin(radian);

	    return  new Vector2D(v.x * cos - v.y * sin,
				v.x * sin + v.y * cos );
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	public static Vector2D interporate(Vector2D start, Vector2D end, double t){
	    return new Vector2D( start.x * (1-t) + end.x * t, start.y * (1-t) + end.y * t);
	}



	
	
	
	
	
	
	
	
	


	public static double distance(int x1, int y1, int x2, int y2){
	  return Math.sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1));
	}
	public static double distance(Point start, Point end){
	  return distance(start.x, start.y, end.x, end.y);
	}
	public static double distance(double x1, double y1, double x2, double y2){
	  return Math.sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1));
	}
	public static Vector2D interporate(Point start, Point end, double t){
	  return new Vector2D( start.x * (1-t) + end.x * t, start.y * (1-t) + end.y * t);
	}
	public Point point(){
		return new Point((int)x, (int)y);
	}





	static public Point translatePoint(Point p, Vector2D vec){
		return new Point((int)(p.x+vec.x), (int)(p.y+vec.y));
	}
}

/*
public class Vector2DD {
	
	

	public double x;
	public double y;
	

	Vector2DD(Point start, Point end){
		x = end.x -  start.x;
		y = end.y -  start.y;
	}
	Vector2DD(Vertex2D start, Vertex2D end){
		x = end.x -  start.x;
		y = end.y -  start.y;
	}

	Vector2DD(Point p){
		x = p.x;
		y = p.y;
	}
	Vector2DD(){
	}
	Vector2DD(double _x, double _y){
		x = _x;
		y = _y;
	}


	public Vector2DD flip_y_axis(){
	    return new Vector2DD(-x, y);
	}
	// rotate 0-360
	public Vector2DD rotate(double degree){
	    if (degree == 90)
		return new Vector2DD(-y, x);
	    else if (degree == 180)
		return new Vector2DD(-x, -y);
	    else if (degree == 270)
		return new Vector2DD(y, -x);

	    
	    double radian = degree * Math.PI / 180.0;
	    double cos = Math.cos(radian);
	    double sin = Math.sin(radian);

	    return  new Vector2DD(x * cos - y * sin,
				x * sin + y * cos );
	}

	// ͕ςȂB
    protected Vector2DD add(Vector2DD v){
		return new Vector2DD(x + v.x, y + v.y);
    }
	public Point add(Point p){
		return new Point((int)x+p.x, (int)y+p.y);	
	}

	// ͕ςȂB
	public Vector2DD normalize(){
	    double length = length();
	    if (length == 0) length = 1;
	    return new Vector2DD(x / length, y / length);
	}	
	public Vector2DD get_normalized(){
	    double length = length();
	    if (length == 0) length = 1;
	    return new Vector2DD(x / length, y / length);
	}
	// gkB͕ςȂB
	public Vector2DD scale(double scale){
	    return new Vector2DD(x * scale, y *scale);
	}
    	public Vector2DD multiple(double m){
	    return new Vector2DD(x * m, y * m);
   	}
    	public void multiple_self(double m){
	    x *= m;
	    y *= m;
   	}
    	public void add_self(Vector2DD v){
	    x += v.x;
	    y += v.y;
   	}
	public void rotate90_self(){
	    double xx = x;
	    double yy = y;
	    x = -y;
	    y = x;	
	}
	public Vector2DD rotate90(){
	    return new Vector2DD(-y, x);
	}
 

	// ς
	public void normalize_self(){
	    double length = length();
	    if (length == 0) length = 1;
	    x = x/length;
	    y = y/length;
	}

	public double length(){
	  return Math.sqrt(x*x + y*y);
	}

	public double inner_product(Vector2DD v){
		return x * v.x + y * v.y;
	}
	public double outer_product(Vector2DD v){
		return x * v.y - y * v.x;
	}
	public double dot_product(Vector2DD v){
		return x * v.x + y * v.y;
	}
	public double cross_product(Vector2DD v){
		return x * v.y - y * v.x;
	}

	public double get_cos(Vector2DD v){
	  double length = 
	    Math.sqrt( (x*x + y*y)*(v.x*v.x + v.y*v.y) );
	  if (length >0)
	    return inner_product(v) / length;
	  else
	    return 0;
	}

	public double get_sin(Vector2DD v){
	  double length = 
	    Math.sqrt( (x*x + y*y)*(v.x*v.x + v.y*v.y) );
	  if (length >0)
	    return cross_product(v) / length;
	  else
	    return 0;
	}  

	public double sin(Vector2DD v){
		return get_sin(v);
	}	
	public double cos(Vector2DD v){
		return get_cos(v);
	}	

   // n_vƂ̋p߂B0-PI Polygon2.set_normal()
    public double get_relative_angle(Vector2DD v){
	double cosine = cos(v);
	if (cosine <= -1) return Math.PI;
	else if (cosine >= 1) return 0;
	else return Math.acos(cos(v));
    }

	// 0-360 SilhuetteFind.cusp 
	// Skeleton  J-loop ɂgB
	public double get_angle(Vector2DD v){
		double cos = get_cos(v);
		double sin = get_sin(v);
		if (cos == 0)
			if (sin > 0)	return 90;
			else		return 270;
		if (sin == 0)
			if (cos > 0)	return 0;
			else		return 180;
		
		// -PI/2   ...   +PI/2
		double angle = 180 * Math.atan(sin/cos) / Math.PI;

		if (cos < 0)
			angle = angle + 180;
		if (angle < 0)
			angle = angle + 360;
		return angle;
	}
		
	// get angel between -180 ... 180
	public double get_angle180(Vector2DD v){
		double cos = get_cos(v);
		double sin = get_sin(v);
		if (cos == 0)
			if (sin > 0)	return 90;
			else		return -90;
		if (sin == 0)
			if (cos > 0)	return 0;
			else		return 180;
		
		// -PI/2   ...   +PI/2
		double angle = 180 * Math.atan(sin/cos) / Math.PI;

		if (cos < 0)
			if (angle>0)
				angle -= 180;
			else
				angle +=180;
		return angle;
	}


	public static double distance(int x1, int y1, int x2, int y2){
	  return Math.sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1));
	}
	public static double distance(Point start, Point end){
	  return distance(start.x, start.y, end.x, end.y);
	}
	public static double distance(double x1, double y1, double x2, double y2){
	  return Math.sqrt((x2-x1)*(x2-x1)+(y2-y1)*(y2-y1));
	}
	public static Vector2DD interporate(Point start, Point end, double t){
	  return new Vector2DD( start.x * (1-t) + end.x * t, start.y * (1-t) + end.y * t);
	}
	public Point point(){
		return new Point((int)x, (int)y);
	}


	public static double distance(Vertex2D start, Vertex2D end){
	  return distance(start.x, start.y, end.x, end.y);
	}

	public Vertex2D vertex2D(){
		return new Vertex2D(x, y);
	}
	
	public static double adjust_angle360(double angle){
		angle = angle - Math.floor(angle/360)*360;
		return angle;
	}

	public static double adjust_angle180(double angle){
		angle = angle - Math.floor(angle/360)*360;
		if (angle>180)
				angle -=360;
		return angle;
	}
	
	static public boolean cross(Point p0, Point p1, Point q0, Point q1){
		Vector2DD vp = new Vector2DD(p0, p1);
		Vector2DD vq = new Vector2DD(q0, q1);
		// v = p0+t*vp
		// v = q0+s*vq
		double d = vp.x * vq.y - vp.y * vq.x;
		double t = (vq.y*(q0.x-p0.x)-vq.x*(q0.y-p0.y) )/d;
		double s = -(-vp.y*(q0.x-p0.x)+vp.x*(q0.y-p0.y) )/d;
		return (s>0 && s<1 && t>0 && t<1);
	}

}
*/