package tools.calc;

//import java.awt.*;
import java.awt.Rectangle;
import java.awt.Point;
import java.util.*;
import board.*;

public class HandCalcMain
{
	
	
	public static String calculate(Vector strokes){
		String string = recognize(strokes);
		
		if (!string.substring(string.length()-1).equals("="))
			return null;
		
		System.out.println(string);
		
		
		double total=0;
		double number=0;
		String operator = "+";
		for(int i=0; i<string.length()-1; i++){
			String c = string.substring(i,i+1);
			if (c.equals("+") || c.equals("-") || c.equals("*") || c.equals("/")){
				total = calculate(operator, total, number);
				operator = c;
				number = 0;
			}
			else {
				number = 10*number + (new Integer(c)).intValue();	
			}
			
		}
		total = calculate(operator, total, number);
		
		System.out.println(" "+total);
		
		String result = ""+total;
		
		if (total - (int)total == 0)
			result = ""+(int)total;
		else if (result.length() > (""+(int)total).length()+2)
			result = result.substring( 0, (""+(int)total).length()+3);
		return result;
	}	
	
	static public double calculate(String c, double total, double number){
		if (c.equals("+")){
			total += number;
		}
		else if (c.equals("-")){
			total -= number;
		}
		else if (c.equals("*")){
			total *= number;
		}
		else if (c.equals("/")){
			total /= number;
		}
		return total;		
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	public static String recognize(Vector strokes){
	
		Vector letters = new Vector();
		
		Stroke prev_stroke = null;
		for(int i=0; i<strokes.size(); i++){
			Stroke stroke = (Stroke)strokes.elementAt(i);
			Rectangle bbox = stroke.bbox;
		
			if (prev_stroke != null){
				Letter letter = combine(prev_stroke, stroke);
				if (letter != null){
					letters.addElement(letter);
					prev_stroke = null;
				}
				else {
					letters.addElement(new Letter(prev_stroke));
					prev_stroke = stroke;
				}
			}
			else
				prev_stroke = stroke;
			
		}
		
		if (prev_stroke != null)
			letters.addElement(new Letter(prev_stroke));
			
		
		
		String result = "";
		for(int i=0; i<letters.size(); i++){
			result += recognize((Letter)letters.elementAt(i));
		}
		
		return result;
	}
	
	
	public static Letter combine(Stroke stroke0, Stroke stroke1){
		Rectangle bbox0 = stroke0.bbox;	
		Rectangle bbox1 = stroke1.bbox;
		
		if (bbox0.intersects(bbox1))
			return new Letter(stroke0, stroke1);
		
		Rectangle bbox = bbox0.union(bbox1);
		
		// "7"
		if (bbox1.width > bbox.width*0.7 && bbox1.height > bbox.height*0.7
			&& bbox0.width < bbox.width*0.3 && bbox0.height < bbox.height*0.5){
			return new Letter(stroke0, stroke1, "7");
		}

		// "="
		if (is_horizontal_line(stroke0) && is_horizontal_line(stroke1))
			return new Letter(stroke0, stroke1, "=");
		
		return null;
		
	}
	
	
	
	
	public static boolean is_horizontal_line(Stroke stroke){
		Rectangle bbox = stroke.bbox;
		if (bbox.width < 2* bbox.height)
			return false;
		
		return is_line(stroke);
	}
	public static boolean is_vertical_line(Stroke stroke){
		Rectangle bbox = stroke.bbox;
		if (bbox.height < 2* bbox.width)
			return false;
		return is_line(stroke);
	}		
	public static boolean is_line(Stroke stroke){
		double l = Vertex2D.distance((Point) stroke.points.elementAt(0),
									 (Point) stroke.points.elementAt(stroke.points.size()-1));
		return (stroke.length() / l < 1.5);
	}
	

	
	// "/"
	public static boolean is_slash(Stroke stroke){
		Rectangle bbox = stroke.bbox;
		if (bbox.height > 2* bbox.width || bbox.width > 2* bbox.height)
			return false;
		if (!is_line(stroke))
			return false;
		
		Point p0 = (Point) stroke.points.elementAt(0);
		Point p1 =  (Point) stroke.points.elementAt(stroke.points.size()-1);
		
		return ((p1.x - p0.x)*(p1.y - p0.y) < 0);
	}		
	// "\"
	public static boolean is_back_slash(Stroke stroke){
		Rectangle bbox = stroke.bbox;
		if (bbox.height > 2* bbox.width || bbox.width > 2* bbox.height)
			return false;
		if (!is_line(stroke))
			return false;
		
		Point p0 = (Point) stroke.points.elementAt(0);
		Point p1 =  (Point) stroke.points.elementAt(stroke.points.size()-1);
		
		return ((p1.x - p0.x)*(p1.y - p0.y) > 0);
	}		

	
	
	
	
	
	
	
	public static String recognize(Letter letter){
		if (letter.strokes.size() == 2)
			return recognize((Stroke)letter.strokes.elementAt(0), 
							 (Stroke)letter.strokes.elementAt(1),
							 letter.string);
		
		return recognize((Stroke)letter.strokes.elementAt(0));
	}
	
	public static String recognize(Stroke stroke){
		if (is_horizontal_line(stroke))
			return "-";
		else if (is_vertical_line(stroke))
			return "1";
		else if (is_slash(stroke))
			return "/";
				   
		String h_motion = get_horizontal_motion(stroke);
		//System.out.println(""+h_motion);
		
		if (h_motion.equals("101"))
			return "2";
		else if (h_motion.equals("1010"))
			return "3";
		else if (h_motion.equals("0101") || h_motion.equals("01010") )
			return "8";
		

		// "6" or "9" or "0"
		else if (h_motion.equals("010")){
			Point p = (Point) stroke.points.elementAt(0);
			if (p.y > stroke.bbox.y + stroke.bbox.height/2)
				return "0";
			
			Point q = (Point) stroke.points.elementAt(stroke.points.size()-1);
			if (q.y == stroke.bbox.height + stroke.bbox.y)
				return "9";
			
			if (q.y > stroke.bbox.y + stroke.bbox.height * 0.3)
				return "6";
			else
				return "0";
		
		}		
		
		
		return "?";
	}
	
	public static String recognize(Stroke stroke0, Stroke stroke1, String string){
		
		// "7" "=" (no intersection)
		if (string != null)
			return string;
		
		
		
		
		if (is_horizontal_line(stroke0) && is_vertical_line(stroke1))
			return "+";
		
		if (is_back_slash(stroke0) && is_slash(stroke1))
			return "*";

		if (is_line(stroke0))
			return "7";
		
		if (is_horizontal_line(stroke1))
			return "5";
		
		return "4";
	}
	
	
	
	
	
	
	
	
	// "1010" = right,left,right,left
	public static String get_horizontal_motion(Stroke stroke){
		stroke = StrokeAnalyzer.resample_by_number(stroke, 8);
		int prev = -1;
		String result = "";
		for(int i=0; i<stroke.points.size()-1; i++){
			Point p = (Point) stroke.points.elementAt(i);
			Point q = (Point) stroke.points.elementAt(i+1);
			int d;
			if (q.x >= p.x)
				d = 1;
			else
				d = 0;
			
			if (prev != d){
				result += d;
			}
			prev = d;
		}
		return result;
	}
	
	
	
}









class Letter {
	Letter(){
		strokes = new Vector();
	}
	String string;
	Vector strokes = new Vector();
	Letter(Stroke stroke){
		strokes.addElement(stroke);
	}
	Letter(Stroke stroke, Stroke stroke2){
		strokes.addElement(stroke);
		strokes.addElement(stroke2);
	}
	Letter(Stroke stroke, Stroke stroke2, String s){
		strokes.addElement(stroke);
		strokes.addElement(stroke2);
		string = s;
	}
	
}































